import logging

import requests
from django.db import IntegrityError, transaction
from django.conf import settings

from dcim.models import Switch, SwitchPort
from inventory.models import InventoryObject, RacksUser


log = logging.getLogger('importers.dcim')


class Session(object):
    def __init__(self):
        self.session = requests.session()
        self.session.auth = (settings.DCIM_USER, settings.DCIM_PASSWORD)

    def request(self, object_type, object_id=None, filters={}):
        if object_id is not None:
            if filters:
                # Code error, will be obvious from traceback
                raise TypeError('Either pass object_id or filters')
            path = '{}/{}'.format(object_type, object_id)
        else:
            path = object_type

        response = self.session.get(settings.DCIM_API + path, params=filters,
                                    timeout=settings.DCIM_API_TIMEOUT)

        assert response.status_code == 200

        # DCIM has a bug, it returns empty list instead of empty dict, handling this case
        return response.json()[object_type] or {}


def import_ports(switch, session):
    deviceports = session.request('deviceport', switch.dcim_id)

    existing_ports = {
        p.number: p
        for p in switch.ports.all()
    }

    for number, deviceport in deviceports.items():
        try:
            port = existing_ports.pop(number)
        except KeyError:
            port = SwitchPort(number=number, switch=switch)

        port.label = deviceport['Label']

        connected_device_dcim_id = deviceport['ConnectedDeviceID']

        if connected_device_dcim_id is not None:
            try:
                server = InventoryObject.objects.get(dcim_id=connected_device_dcim_id)
            except InventoryObject.DoesNotExist:
                server = None

            if server:
                # connected_device_dcim_id is server id, assign it to port
                port.dcim_server_id = connected_device_dcim_id
            else:
                # Otherwise it's something else, set to None
                port.dcim_server_id = None
        else:
            port.dcim_server = None

        port.save()

    # Delete ports in database but not in DCIM
    for port in existing_ports.values():
        port.delete()


@transaction.commit_on_success
def import_switches():
    session = Session()

    devices = session.request('device', filters={
        'DeviceType': 'Switch',
    })

    existing_switches = {
        s.dcim_id: s
        for s in Switch.objects.all()
    }

    for device in devices:
        dcim_id = device['DeviceID']

        try:
            switch = existing_switches.pop(dcim_id)
        except KeyError:
            switch = Switch(dcim_id=dcim_id)

        switch.label = device['Label']
        switch.management_address = device['PrimaryIP']

        if not switch.management_address:
            log.error("Switch '%s' does not have valid PrimaryIP field", switch.label)
            continue

        try:
            switch.save()
        except IntegrityError:
            log.exception("Switch id: '%s', label: '%s' save to DB failed",
                          device['DeviceID'], device['Label'])
            continue

        try:
            import_ports(switch, session)
        except:
            log.exception("Ports import failed for switch %s", switch)
            raise

    # Delete switches in database but not in DCIM
    for s in existing_switches.values():
        s.delete()


@transaction.commit_on_success
def import_servers():
    filters = {
        'DeviceType': 'Server',
    }

    if settings.DCIM_SERVER_OWNER_IDS_LIST:
        owners = settings.DCIM_SERVER_OWNER_IDS_LIST
    elif settings.DCIM_SERVER_OWNER_ID is not None:
        # if single owner, for backward compatibility
        owners = list(settings.DCIM_SERVER_OWNER_ID)
    else:
        owners = []  # any owner

    existing_servers = {
        s.dcim_id: s
        for s in InventoryObject.objects.filter(dcim_id__isnull=False)
    }

    # Build { full_name: user } dict for all active RacksUser's
    racks_users = {
        u' '.join((user.first_name, user.last_name)): user for user in RacksUser.objects.all()
    }

    session = Session()

    # if we have owners apply filter, otherwise fetch all devices
    if owners:
        devices = []
        for owner_id in owners:
            filters['Owner'] = owner_id
            devices += session.request('device', filters=filters)
    else:
        devices = session.request('device', filters=filters)

    for device in devices:
        dcim_id = device['DeviceID']

        try:
            server = existing_servers.pop(dcim_id)
        except KeyError:
            server = InventoryObject(dcim_id=dcim_id)

        server.dcim_label = device['Label']

        # Need or here, because DCIM return empty CustomValues as []
        custom_values = device.get('CustomValues') or {}

        if settings.DCIM_JIRA_ISSUE_CUSTOM_VALUE_ID is not None:
            server.dcim_jira_issue = custom_values.get(str(settings.DCIM_JIRA_ISSUE_CUSTOM_VALUE_ID))

        # If no user or group are assigned to this server, we try to auto-assign it to reporter.
        if (server.assigned_user is None) and (server.assigned_group is None):
            reporter = custom_values.get(str(settings.DCIM_CUSTOM_REPORTER_VALUE_ID), None)
            if not reporter:
                log.warning('Failed to auto-assign server `%s`: no reporter.', dcim_id)
            elif reporter in racks_users:
                server.assigned_user = racks_users[reporter]
                log.info('Server `%s` assigned to user `%s`.',
                         dcim_id, reporter)
            else:
                log.warning('Failed to auto-assign server `%s`: reporter `%s` not found.',
                            dcim_id, reporter)

        server.save()

    # Orphan object no longer in DCIM
    for s in existing_servers.values():
        s.dcim_orphan()

    InventoryObject.prune_orphaned_objects()
