import csv

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.shortcuts import render
from django.shortcuts import HttpResponse

from inventory.models import (
    InventoryObject,
    InventoryObjectAttribute,
    InventoryObjectLabel,
)


@login_required
def labels(request):
    labels = InventoryObjectLabel.objects.all().order_by('name')

    return render(request, 'inventory/labels.html', {
        'labels': labels,
    })


@login_required
def search(request):
    query = request.GET.get('query').strip()
    user = request.racks_user

    if query.startswith('^') and query.endswith('$'):
        search_type = 'iexact'
        query = query[1:-1]
    elif query.startswith('^'):
        search_type = 'istartswith'
        query = query[1:]
    elif query.endswith('$'):
        search_type = 'iendswith'
        query = query[:-1]
    else:
        search_type = 'icontains'

    attributes = _search_for_inventory_objects(search_type, query, user)
    object_attrs = _search_for_inventory_object_attributes(search_type, query)

    for name in object_attrs:
        if name in attributes:
            attributes[name]['results'] += object_attrs[name]['results']
        else:
            attributes[name] = object_attrs[name]

    return render(request, 'inventory/search.html', {
        'attributes': attributes,
        'query': query,
    })


@login_required
def export_object_inventory(request):
    label = request.GET.get('label')
    responsible = request.GET.get('responsible')

    filters = {}
    if label is not None:
        filters['label'] = InventoryObjectLabel.objects.get(name=label)

    if responsible is not None:
        filters['responsible'] = User.objects.get(username=responsible)

    object_list = InventoryObject.objects.exclude(name=None).filter(**filters)

    columns = [
        'name',
        'location_name',
        'location_fact',
        'hardware',
        'type',
        'label_string',
        'jira_issue',
        'dcim_jira_issue',
        'responsible_string',
        'assigned_user_name',
        'assigned_group_name',
    ]

    response = HttpResponse(
        content_type='text/csv',
        reason='Ok',
    )
    writer = csv.writer(response)
    writer.writerow(columns)
    for i in object_list:
        writer.writerow([getattr(i, c) for c in columns])

    return response


def _search_for_inventory_objects(search_type, query, user):
    fields_mapping = InventoryObject.SEARCHABLE_FIELDS_MAPPING.items()
    attributes = {}

    for display_name, attr_name in fields_mapping:
        search_query = {'{}__{}'.format(attr_name, search_type): query}
        inventory_objs = [
            obj for obj in InventoryObject.objects.filter(**search_query)
            if InventoryObject.get_permissions(user, obj).get_field_permissions(
                attr_name.split('__')[0]).view
        ]

        for inventory_object in inventory_objs:
            value = inventory_object

            for child in attr_name.split('__'):
                value = getattr(value, child)

            data = {
                'name': display_name,
                'value': value,
            }
            name = inventory_object.display_name
            if name in attributes:
                attributes[name]['results'].append(data)
            else:
                attributes[name] = {
                    'object': inventory_object,
                    'results': [data],
                }

    return attributes


def _search_for_inventory_object_attributes(search_type, query):
    search_query = {'value__{}'.format(search_type): query}
    attributes = {}

    inventory_attributes_objs = InventoryObjectAttribute.objects.\
        filter(**search_query).\
        select_related('inventory_object')

    for attribute in inventory_attributes_objs:
        _obj = attribute.object

        attributes[_obj.display_name] = {
            'object': _obj,
            'results': [{
                'name': attribute.name,
                'value': attribute.awesome_value,
            }],
        }

    return attributes
